# -*- coding: utf-8 -*-
'''
Dynamic roster from terraform current state
===========================================

This roster module allows you dynamically generate the roster from the terraform
resources defined with the `Terraform Salt`_ provider.

It exposes all salt_host resources with the same attributes to the salt-ssh
roster, making it completely independent of the type of terraform resource, and
providing the integration using terraform constructs with interpolation.

Basic Example
-------------

Given a simple salt-ssh tree with a Saltfile:

.. code-block:: yaml

    salt-ssh:
      config_dir: etc/salt
      max_procs: 30
      wipe_ssh: True

and ``etc/salt/master``:

.. code-block:: yaml

    root_dir: .
      file_roots:
        base:
          - srv/salt
      pillar_roots:
        base:
          - srv/pillar
      roster: terraform

In the same folder as your ``Saltfile``, create terraform file with resources
like cloud instances, virtual machines, etc. For every single one of those that
you want to manage with Salt, create a ``salt_host`` resource:

.. code-block:: text

    resource "salt_host" "dbminion" {
      salt_id = "dbserver"
      host = "${libvirt_domain.vm-db.network_interface.0.addresses.0}"
      user = "root"
      passwd = "linux"
    }

You can use the count attribute to create multiple roster entries with a single
definition. Please refer to the `Terraform Salt`_ provider for more detailed
examples.

.. _Terraform Salt: https://github.com/dmacvicar/terraform-provider-salt
'''
# Import Python libs
from __future__ import absolute_import, unicode_literals
import logging
import os.path

# Import Salt libs
import salt.utils.files
import salt.utils.json

log = logging.getLogger(__name__)

TF_OUTPUT_PREFIX = 'salt.roster.'
TF_ROSTER_ATTRS = {'host': 's',
                   'user': 's',
                   'passwd': 's',
                   'port': 'i',
                   'sudo': 'b',
                   'sudo_user': 's',
                   'tty': 'b', 'priv': 's',
                   'timeout': 'i',
                   'minion_opts': 'm',
                   'thin_dir': 's',
                   'cmd_umask': 'i'}
MINION_ID = 'salt_id'


def _handle_salt_host_resource(resource):
    '''
    Handles salt_host resources.
    See https://github.com/dmacvicar/terraform-provider-salt

    Returns roster attributes for the resource or None
    '''
    ret = {}
    attrs = resource.get('primary', {}).get('attributes', {})
    ret[MINION_ID] = attrs.get(MINION_ID)
    valid_attrs = set(attrs.keys()).intersection(TF_ROSTER_ATTRS.keys())
    for attr in valid_attrs:
        ret[attr] = _cast_output_to_type(attrs.get(attr), TF_ROSTER_ATTRS.get(attr))
    return ret


def _add_ssh_key(ret):
    '''
    Setups the salt-ssh minion to be accessed with salt-ssh default key
    '''
    priv = None
    if __opts__.get('ssh_use_home_key') and os.path.isfile(os.path.expanduser('~/.ssh/id_rsa')):
        priv = os.path.expanduser('~/.ssh/id_rsa')
    else:
        priv = __opts__.get(
            'ssh_priv',
            os.path.abspath(os.path.join(
                __opts__['pki_dir'],
                'ssh',
                'salt-ssh.rsa'
            ))
        )
    if priv and os.path.isfile(priv):
        ret['priv'] = priv


def _cast_output_to_type(value, typ):
    '''cast the value depending on the terraform type'''
    if typ == 'b':
        return bool(value)
    if typ == 'i':
        return int(value)
    return value


def _parse_state_file(state_file_path='terraform.tfstate'):
    '''
    Parses the terraform state file passing different resource types to the right handler
    '''
    ret = {}
    with salt.utils.files.fopen(state_file_path, 'r') as fh_:
        tfstate = salt.utils.json.load(fh_)

    modules = tfstate.get('modules')
    if not modules:
        log.error('Malformed tfstate file. No modules found')
        return ret

    for module in modules:
        resources = module.get('resources', [])
        for resource_name, resource in salt.ext.six.iteritems(resources):
            roster_entry = None
            if resource['type'] == 'salt_host':
                roster_entry = _handle_salt_host_resource(resource)

            if not roster_entry:
                continue

            minion_id = roster_entry.get(MINION_ID, resource.get('id'))
            if not minion_id:
                continue

            if MINION_ID in roster_entry:
                del roster_entry[MINION_ID]
            _add_ssh_key(roster_entry)
            ret[minion_id] = roster_entry
    return ret


def targets(tgt, tgt_type='glob', **kwargs):  # pylint: disable=W0613
    '''
    Returns the roster from the terraform state file, checks opts for location, but defaults to terraform.tfstate
    '''
    roster_file = os.path.abspath('terraform.tfstate')
    if __opts__.get('roster_file'):
        roster_file = os.path.abspath(__opts__['roster_file'])

    if not os.path.isfile(roster_file):
        log.error("Can't find terraform state file '%s'", roster_file)
        return {}

    log.debug('terraform roster: using %s state file', roster_file)

    if not roster_file.endswith('.tfstate'):
        log.error("Terraform roster can only be used with terraform state files")
        return {}

    raw = _parse_state_file(roster_file)
    log.debug('%s hosts in terraform state file', len(raw))
    return __utils__['roster_matcher.targets'](raw, tgt, tgt_type, 'ipv4')



# -*- coding: utf-8 -*-
'''
The match module allows for match routines to be run and determine target specs
'''
from __future__ import absolute_import, print_function, unicode_literals

# Import python libs
import inspect
import logging
import sys
import copy

# Import salt libs
import salt.minion
import salt.loader
from salt.defaults import DEFAULT_TARGET_DELIM
from salt.ext import six

__func_alias__ = {
    'list_': 'list'
}

log = logging.getLogger(__name__)


def compound(tgt, minion_id=None):
    '''
    Return True if the minion ID matches the given compound target

    minion_id
        Specify the minion ID to match against the target expression

        .. versionadded:: 2014.7.0

    CLI Example:

    .. code-block:: bash

        salt '*' match.compound 'L@cheese,foo and *'
    '''
    if minion_id is not None:
        opts = copy.copy(__opts__)
        if not isinstance(minion_id, six.string_types):
            minion_id = six.text_type(minion_id)
        opts['id'] = minion_id
    else:
        opts = __opts__
    matchers = salt.loader.matchers(opts)
    try:
        return matchers['compound_match.match'](tgt)
    except Exception as exc:
        log.exception(exc)
        return False


def ipcidr(tgt):
    '''
    Return True if the minion matches the given ipcidr target

    CLI Example:

    .. code-block:: bash

        salt '*' match.ipcidr '192.168.44.0/24'

    delimiter
    Pillar Example:

    .. code-block:: yaml

       '172.16.0.0/12':
         - match: ipcidr
         - nodeclass: internal

    '''
    matchers = salt.loader.matchers(__opts__)
    try:
        return matchers['ipcidr_match.match'](tgt)
    except Exception as exc:
        log.exception(exc)
        return False


def pillar_pcre(tgt, delimiter=DEFAULT_TARGET_DELIM):
    '''
    Return True if the minion matches the given pillar_pcre target. The
    ``delimiter`` argument can be used to specify a different delimiter.

    CLI Example:

    .. code-block:: bash

        salt '*' match.pillar_pcre 'cheese:(swiss|american)'
        salt '*' match.pillar_pcre 'clone_url|https://github\\.com/.*\\.git' delimiter='|'

    delimiter
        Specify an alternate delimiter to use when traversing a nested dict

        .. versionadded:: 2014.7.0

    delim
        Specify an alternate delimiter to use when traversing a nested dict

        .. versionadded:: 0.16.4
        .. deprecated:: 2015.8.0
    '''
    matchers = salt.loader.matchers(__opts__)
    try:
        return matchers['pillar_pcre_match.match'](tgt, delimiter=delimiter)
    except Exception as exc:
        log.exception(exc)
        return False


def pillar(tgt, delimiter=DEFAULT_TARGET_DELIM):
    '''
    Return True if the minion matches the given pillar target. The
    ``delimiter`` argument can be used to specify a different delimiter.

    CLI Example:

    .. code-block:: bash

        salt '*' match.pillar 'cheese:foo'
        salt '*' match.pillar 'clone_url|https://github.com/saltstack/salt.git' delimiter='|'

    delimiter
        Specify an alternate delimiter to use when traversing a nested dict

        .. versionadded:: 2014.7.0

    delim
        Specify an alternate delimiter to use when traversing a nested dict

        .. versionadded:: 0.16.4
        .. deprecated:: 2015.8.0
    '''
    matchers = salt.loader.matchers(__opts__)
    try:
        return matchers['pillar_match.match'](tgt, delimiter=delimiter)
    except Exception as exc:
        log.exception(exc)
        return False


def data(tgt):
    '''
    Return True if the minion matches the given data target

    CLI Example:

    .. code-block:: bash

        salt '*' match.data 'spam:eggs'
    '''
    matchers = salt.loader.matchers(__opts__)
    try:
        return matchers['data_match.match'](tgt)
    except Exception as exc:
        log.exception(exc)
        return False


def grain_pcre(tgt, delimiter=DEFAULT_TARGET_DELIM):
    '''
    Return True if the minion matches the given grain_pcre target. The
    ``delimiter`` argument can be used to specify a different delimiter.

    CLI Example:

    .. code-block:: bash

        salt '*' match.grain_pcre 'os:Fedo.*'
        salt '*' match.grain_pcre 'ipv6|2001:.*' delimiter='|'

    delimiter
        Specify an alternate delimiter to use when traversing a nested dict

        .. versionadded:: 2014.7.0

    delim
        Specify an alternate delimiter to use when traversing a nested dict

        .. versionadded:: 0.16.4
        .. deprecated:: 2015.8.0
    '''
    matchers = salt.loader.matchers(__opts__)
    try:
        return matchers['grain_pcre_match.match'](tgt, delimiter=delimiter)
    except Exception as exc:
        log.exception(exc)
        return False


def grain(tgt, delimiter=DEFAULT_TARGET_DELIM):
    '''
    Return True if the minion matches the given grain target. The ``delimiter``
    argument can be used to specify a different delimiter.

    CLI Example:

    .. code-block:: bash

        salt '*' match.grain 'os:Ubuntu'
        salt '*' match.grain 'ipv6|2001:db8::ff00:42:8329' delimiter='|'

    delimiter
        Specify an alternate delimiter to use when traversing a nested dict

        .. versionadded:: 2014.7.0

    delim
        Specify an alternate delimiter to use when traversing a nested dict

        .. versionadded:: 0.16.4
        .. deprecated:: 2015.8.0
    '''
    matchers = salt.loader.matchers(__opts__)
    try:
        return matchers['grain_match.match'](tgt, delimiter=delimiter)
    except Exception as exc:
        log.exception(exc)
        return False


def list_(tgt, minion_id=None):
    '''
    Return True if the minion ID matches the given list target

    minion_id
        Specify the minion ID to match against the target expression

        .. versionadded:: 2014.7.0

    CLI Example:

    .. code-block:: bash

        salt '*' match.list 'server1,server2'
    '''
    if minion_id is not None:
        opts = copy.copy(__opts__)
        if not isinstance(minion_id, six.string_types):
            minion_id = six.text_type(minion_id)
        opts['id'] = minion_id
    else:
        opts = __opts__
    matchers = salt.loader.matchers(opts)
    try:
        return matchers['list_match.match'](tgt)
    except Exception as exc:
        log.exception(exc)
        return False


def pcre(tgt, minion_id=None):
    '''
    Return True if the minion ID matches the given pcre target

    minion_id
        Specify the minion ID to match against the target expression

        .. versionadded:: 2014.7.0

    CLI Example:

    .. code-block:: bash

        salt '*' match.pcre '.*'
    '''
    if minion_id is not None:
        opts = copy.copy(__opts__)
        if not isinstance(minion_id, six.string_types):
            minion_id = six.text_type(minion_id)
        opts['id'] = minion_id
    else:
        opts = __opts__
    matchers = salt.loader.matchers(opts)
    try:
        return matchers['pcre_match.match'](tgt)
    except Exception as exc:
        log.exception(exc)
        return False


def glob(tgt, minion_id=None):
    '''
    Return True if the minion ID matches the given glob target

    minion_id
        Specify the minion ID to match against the target expression

        .. versionadded:: 2014.7.0

    CLI Example:

    .. code-block:: bash

        salt '*' match.glob '*'
    '''
    if minion_id is not None:
        opts = copy.copy(__opts__)
        if not isinstance(minion_id, six.string_types):
            minion_id = six.text_type(minion_id)
        opts['id'] = minion_id
    else:
        opts = __opts__
    matchers = salt.loader.matchers(opts)

    try:
        return matchers['glob_match.match'](tgt)
    except Exception as exc:
        log.exception(exc)
        return False


def filter_by(lookup,
              tgt_type='compound',
              minion_id=None,
              default='default'):
    '''
    Return the first match in a dictionary of target patterns

    .. versionadded:: 2014.7.0

    CLI Example:

    .. code-block:: bash

        salt '*' match.filter_by '{foo*: Foo!, bar*: Bar!}' minion_id=bar03

    Pillar Example:

    .. code-block:: jinja

        # Filter the data for the current minion into a variable:
        {% set roles = salt['match.filter_by']({
            'web*': ['app', 'caching'],
            'db*': ['db'],
        }, default='web*') %}

        # Make the filtered data available to Pillar:
        roles: {{ roles | yaml() }}
    '''
    expr_funcs = dict(inspect.getmembers(sys.modules[__name__],
        predicate=inspect.isfunction))

    for key in lookup:
        params = (key, minion_id) if minion_id else (key, )
        if expr_funcs[tgt_type](*params):
            return lookup[key]

    return lookup.get(default, None)


def search_by(lookup, tgt_type='compound', minion_id=None):
    '''
    Search a dictionary of target strings for matching targets

    This is the inverse of :py:func:`match.filter_by
    <salt.modules.match.filter_by>` and allows matching values instead of
    matching keys. A minion can be matched by multiple entries.

    .. versionadded:: 2017.7.0

    CLI Example:

    .. code-block:: bash

        salt '*' match.search_by '{web: [node1, node2], db: [node2, node]}'

    Pillar Example:

    .. code-block:: jinja

        {% set roles = salt.match.search_by({
            'web': ['G@os_family:Debian not nodeX'],
            'db': ['L@node2,node3 and G@datacenter:west'],
            'caching': ['node3', 'node4'],
        }) %}

        # Make the filtered data available to Pillar:
        roles: {{ roles | yaml() }}
    '''
    expr_funcs = dict(inspect.getmembers(sys.modules[__name__],
        predicate=inspect.isfunction))

    matches = []
    for key, target_list in lookup.items():
        for target in target_list:
            params = (target, minion_id) if minion_id else (target, )
            if expr_funcs[tgt_type](*params):
                matches.append(key)

    return matches or None



# -*- coding: utf-8 -*-
'''
:codeauthor: Shane Lee <slee@saltstack.com>
'''
# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function
import os

# Import Salt Testing Libs
from tests.support.unit import TestCase, skipIf
from tests.support.mock import patch, NO_MOCK, NO_MOCK_REASON

# Import Salt Libs
import salt.modules.win_file as win_file
from salt.exceptions import CommandExecutionError
import salt.utils.platform
import salt.utils.win_dacl


@skipIf(NO_MOCK, NO_MOCK_REASON)
class WinFileTestCase(TestCase):
    '''
        Test cases for salt.modules.win_file
    '''
    FAKE_RET = {'fake': 'ret data'}
    if salt.utils.platform.is_windows():
        FAKE_PATH = os.sep.join(['C:', 'path', 'does', 'not', 'exist'])
    else:
        FAKE_PATH = os.sep.join(['path', 'does', 'not', 'exist'])

    def test_issue_43328_stats(self):
        '''
        Make sure that a CommandExecutionError is raised if the file does NOT
        exist
        '''
        with patch('os.path.exists', return_value=False):
            self.assertRaises(CommandExecutionError,
                              win_file.stats,
                              self.FAKE_PATH)

    def test_issue_43328_check_perms_no_ret(self):
        '''
        Make sure that a CommandExecutionError is raised if the file does NOT
        exist
        '''
        with patch('os.path.exists', return_value=False):
            self.assertRaises(
                CommandExecutionError, win_file.check_perms, self.FAKE_PATH)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jakub Sliva <jakub.sliva@ultimum.io>`
'''

# Import Python Libs
from __future__ import absolute_import
from __future__ import unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

import salt.states.zabbix_template as zabbix_template


INPUT_PARAMS = {"applications": [{"name": "Ceph OSD"}]}

DEFINED_OBJ = {"macros": [{"macro": "{$CEPH_CLUSTER_NAME}", "value": "ceph"}], "host": "A Testing Template",
               "hosts": [{"hostid": "10112"}, {"hostid": "10113"}], "description": "Template for Ceph nodes",
               "groups": [{"groupid": "1"}]}

DEFINED_C_LIST_SUBS = {"applications": [{"name": "Ceph OSD"}], 'graphs': [], 'triggers': [], 'items': [],
                       'httpTests': [], 'screens': [], 'gitems': [], 'discoveries': []}

SUBSTITUTE_PARAMS_CREATE = [DEFINED_OBJ, [], DEFINED_C_LIST_SUBS['applications'], [], [], [], [], [], [], [], []]

EXISTING_OBJ = [{"available": "0", "tls_connect": "1", "maintenance_type": "0", "groups": [{"groupid": "1"}],
                 "macros": [{"macro": "{$CEPH_CLUSTER_NAME}", "hostmacroid": "60", "hostid": "10206", "value": "ceph"}],
                 "hosts": [{"hostid": "10112"}, {"hostid": "10113"}], "status": "3",
                 "description": "Template for Ceph nodes", "host": "A Testing Template", "disable_until": "0",
                 "templateid": "10206", "name": "A Testing Template"}]

SUBSTITUTE_PARAMS_EXISTS = [DEFINED_OBJ, EXISTING_OBJ[0], [], [], [], [], [], [], [], []]

EXISTING_OBJ_DIFF = [{"groups": [{"groupid": "1"}], "macros": [{"macro": "{$CEPH_CLUSTER_NAME}", "hostmacroid": "60",
                                                               "hostid": "10206", "value": "ceph"}],
                      "hosts": [{"hostid": "10112"}, {"hostid": "10113"}], "status": "3", "templateid": "10206",
                      "name": "A Testing Template"}]

SUBSTITUTE_PARAMS_UPDATE = [DEFINED_OBJ, EXISTING_OBJ_DIFF[0], [], [], [], [], [], [], [], []]

DIFF_PARAMS = {'old': {}, 'new': {'macros': [], 'templateid': '10206'}}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ZabbixTemplateTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.zabbix
    '''
    def setup_loader_modules(self):
        return {zabbix_template: {}}

    @patch('salt.states.zabbix_template.CHANGE_STACK', [])
    def test_present_create(self):
        '''
        Test to ensure that named template is created
        '''
        name = 'A Testing Template'
        ret = {'name': name, 'result': False, 'comment': '', 'changes': {}}

        def side_effect_run_query(*args):
            '''
            Differentiate between __salt__ exec module function calls with different parameters.
            '''
            if args[0] == 'template.get':
                return []
            elif args[0] == 'template.create':
                return {'templateids': ['10206']}
            elif args[0] == 'application.get':
                return []
            elif args[0] == 'application.create':
                return {"applicationids": ["701"]}

        with patch.dict(zabbix_template.__opts__, {'test': False}):
            with patch.dict(zabbix_template.__salt__,
                            {'zabbix.get_zabbix_id_mapper': MagicMock(return_value={'template': 'templateid'}),
                             'zabbix.substitute_params': MagicMock(side_effect=SUBSTITUTE_PARAMS_CREATE),
                             'zabbix.run_query': MagicMock(side_effect=side_effect_run_query),
                             'zabbix.compare_params': MagicMock(return_value={})}):
                ret['result'] = True
                ret['comment'] = 'Zabbix Template "{0}" created.'.format(name)
                ret['changes'] = {name: {'old': 'Zabbix Template "{0}" did not exist.'.format(name),
                                         'new': 'Zabbix Template "{0}" created according definition.'.format(name)}}
                self.assertDictEqual(zabbix_template.present(name, {}), ret)

    @patch('salt.states.zabbix_template.CHANGE_STACK', [])
    def test_present_exists(self):
        '''
        Test to ensure that named template is present and not changed
        '''
        name = 'A Testing Template'
        ret = {'name': name, 'result': False, 'comment': '', 'changes': {}}

        def side_effect_run_query(*args):
            '''
            Differentiate between __salt__ exec module function calls with different parameters.
            '''
            if args[0] == 'template.get':
                return EXISTING_OBJ
            elif args[0] == 'application.get':
                return ['non-empty']

        with patch.dict(zabbix_template.__opts__, {'test': False}):
            with patch.dict(zabbix_template.__salt__,
                            {'zabbix.get_zabbix_id_mapper': MagicMock(return_value={'template': 'templateid'}),
                             'zabbix.substitute_params': MagicMock(side_effect=SUBSTITUTE_PARAMS_EXISTS),
                             'zabbix.run_query': MagicMock(side_effect=side_effect_run_query),
                             'zabbix.compare_params': MagicMock(return_value={'new': {}, 'old': {}})}):
                ret['result'] = True
                ret['comment'] = 'Zabbix Template "{0}" already exists and corresponds to a definition.'.format(name)
                self.assertDictEqual(zabbix_template.present(name, {}), ret)

    @patch('salt.states.zabbix_template.CHANGE_STACK', [])
    def test_present_update(self):
        '''
        Test to ensure that named template is present but must be updated
        '''
        name = 'A Testing Template'
        ret = {'name': name, 'result': False, 'comment': '', 'changes': {}}

        def side_effect_run_query(*args):
            '''
            Differentiate between __salt__ exec module function calls with different parameters.
            '''
            if args[0] == 'template.get':
                return ['length of result is 1 = template exists']
            elif args[0] == 'template.update':
                return DIFF_PARAMS

        with patch.dict(zabbix_template.__opts__, {'test': False}):
            with patch.dict(zabbix_template.__salt__,
                            {'zabbix.get_zabbix_id_mapper': MagicMock(return_value={'template': 'templateid'}),
                             'zabbix.substitute_params': MagicMock(side_effect=SUBSTITUTE_PARAMS_UPDATE),
                             'zabbix.run_query': MagicMock(side_effect=side_effect_run_query),
                             'zabbix.compare_params': MagicMock(return_value=DIFF_PARAMS)}):
                ret['result'] = True
                ret['comment'] = 'Zabbix Template "{0}" updated.'.format(name)
                ret['changes'] = {name: {'old': 'Zabbix Template "{0}" differed.'.format(name),
                                         'new': 'Zabbix Template "{0}" updated according definition.'.format(name)}}
                self.assertDictEqual(zabbix_template.present(name, {}), ret)

    def test_absent_test_mode(self):
        '''
        Test to ensure that named template is absent in test mode
        '''
        name = 'A Testing Template'
        ret = {'name': name, 'result': False, 'comment': '', 'changes': {}}
        with patch.dict(zabbix_template.__opts__, {'test': True}):
            with patch.dict(zabbix_template.__salt__, {'zabbix.get_object_id_by_params': MagicMock(return_value=11)}):
                ret['result'] = True
                ret['comment'] = 'Zabbix Template "{0}" would be deleted.'.format(name)
                ret['changes'] = {name: {'old': 'Zabbix Template "{0}" exists.'.format(name),
                                         'new': 'Zabbix Template "{0}" would be deleted.'.format(name)}}
                self.assertDictEqual(zabbix_template.absent(name), ret)

    def test_absent(self):
        '''
        Test to ensure that named template is absent
        '''
        name = 'A Testing Template'
        ret = {'name': name, 'result': False, 'comment': '', 'changes': {}}
        with patch.dict(zabbix_template.__opts__, {'test': False}):
            with patch.dict(zabbix_template.__salt__,
                            {'zabbix.get_object_id_by_params': MagicMock(return_value=False)}):
                ret['result'] = True
                ret['comment'] = 'Zabbix Template "{0}" does not exist.'.format(name)
                self.assertDictEqual(zabbix_template.absent(name), ret)

            with patch.dict(zabbix_template.__salt__, {'zabbix.get_object_id_by_params': MagicMock(return_value=11)}):
                with patch.dict(zabbix_template.__salt__, {'zabbix.run_query': MagicMock(return_value=True)}):
                    ret['result'] = True
                    ret['comment'] = 'Zabbix Template "{0}" deleted.'.format(name)
                    ret['changes'] = {name: {'old': 'Zabbix Template "{0}" existed.'.format(name),
                                             'new': 'Zabbix Template "{0}" deleted.'.format(name)}}
                    self.assertDictEqual(zabbix_template.absent(name), ret)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import
import textwrap

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)

# Import Salt Libs
import salt.modules.pkgng as pkgng


@skipIf(NO_MOCK, NO_MOCK_REASON)
class PkgNgTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.pkgng
    '''

    @classmethod
    def setup_loader_modules(cls):
        return {
            pkgng: {
                '__salt__': {}
            }
        }

    def test_lock(self):
        '''
        Test pkgng.lock
        '''
        lock_cmd = MagicMock(return_value={
            'stdout': ('pkga-1.0\n'
                       'pkgb-2.0\n'),
            'retcode': 0
        })
        with patch.dict(pkgng.__salt__, {'cmd.run_all': lock_cmd}):

            result = pkgng.lock('pkga')
            self.assertTrue(result)
            lock_cmd.assert_called_with(
                ['pkg', 'lock', '-y', '--quiet', '--show-locked', 'pkga'],
                output_loglevel='trace', python_shell=False
            )

            result = pkgng.lock('dummy')
            self.assertFalse(result)
            lock_cmd.assert_called_with(
                ['pkg', 'lock', '-y', '--quiet', '--show-locked', 'dummy'],
                output_loglevel='trace', python_shell=False
            )

    def test_unlock(self):
        '''
        Test pkgng.unlock
        '''
        unlock_cmd = MagicMock(return_value={
            'stdout': ('pkga-1.0\n'
                       'pkgb-2.0\n'),
            'retcode': 0
        })
        with patch.dict(pkgng.__salt__, {'cmd.run_all': unlock_cmd}):

            result = pkgng.unlock('pkga')
            self.assertFalse(result)
            unlock_cmd.assert_called_with(
                ['pkg', 'unlock', '-y', '--quiet', '--show-locked', 'pkga'],
                output_loglevel='trace', python_shell=False
            )

            result = pkgng.unlock('dummy')
            self.assertTrue(result)
            unlock_cmd.assert_called_with(
                ['pkg', 'unlock', '-y', '--quiet', '--show-locked', 'dummy'],
                output_loglevel='trace', python_shell=False
            )

    def test_locked(self):
        '''
        Test pkgng.unlock
        '''
        lock_cmd = MagicMock(return_value={
            'stdout': ('pkga-1.0\n'
                       'pkgb-2.0\n'),
            'retcode': 0
        })
        with patch.dict(pkgng.__salt__, {'cmd.run_all': lock_cmd}):

            result = pkgng.locked('pkga')
            self.assertTrue(result)
            lock_cmd.assert_called_with(
                ['pkg', 'lock', '-y', '--quiet', '--show-locked'],
                output_loglevel='trace', python_shell=False
            )

            result = pkgng.locked('dummy')
            self.assertFalse(result)
            lock_cmd.assert_called_with(
                ['pkg', 'lock', '-y', '--quiet', '--show-locked'],
                output_loglevel='trace', python_shell=False
            )

    def test_list_upgrades_present(self):
        '''
        Test pkgng.list_upgrades with upgrades available
        '''
        pkg_cmd = MagicMock(return_value=textwrap.dedent(
            """
            The following 6 package(s) will be affected (of 0 checked):

            Installed packages to be UPGRADED:
                    pkga: 1.0 -> 1.1
                    pkgb: 2.0 -> 2.1 [FreeBSD]
                    pkgc: 3.0 -> 3.1 [FreeBSD] (dependency changed)
                    pkgd: 4.0 -> 4.1 (dependency changed)

            New packages to be INSTALLED:
                    pkge: 1.0
                    pkgf: 2.0 [FreeBSD]
                    pkgg: 3.0 [FreeBSD] (dependency changed)
                    pkgh: 4.0 (dependency changed)

            Installed packages to be REINSTALLED:
                    pkgi-1.0
                    pkgj-2.0 [FreeBSD]
                    pkgk-3.0 [FreeBSD] (direct dependency changed: pkga)
                    pkgl-4.0 (direct dependency changed: pkgb)

            Installed packages to be DOWNGRADED:
                    pkgm: 1.1 -> 1.0
                    pkgn: 2.1 -> 2.0 [FreeBSD]
                    pkgo: 3.1 -> 3.0 [FreeBSD] (dependency changed)
                    pkgp: 4.1 -> 4.0 (dependency changed)

            Installed packages to be REMOVED:
                    pkgq-1.0
                    pkgr-2.0 [FreeBSD]
                    pkgs-3.0 [FreeBSD] (direct dependency changed: pkga)
                    pkgt-4.0 (direct dependency changed: pkgb)

            Number of packages to be upgraded: 2
            Number of packages to be reinstalled: 2

            The process will require 14 MiB more space.
            22 MiB to be downloaded.
            """
        ))

        with patch.dict(pkgng.__salt__, {'cmd.run_stdout': pkg_cmd}):

            result = pkgng.list_upgrades(refresh=False)
            self.assertDictEqual(result, {'pkga': '1.1', 'pkgb': '2.1', 'pkgc': '3.1', 'pkgd': '4.1'})
            pkg_cmd.assert_called_with(
                ['pkg', 'upgrade', '--dry-run', '--quiet', '--no-repo-update'],
                output_loglevel='trace', python_shell=False, ignore_retcode=True
            )

    def test_list_upgrades_absent(self):
        '''
        Test pkgng.list_upgrades with no upgrades available
        '''
        pkg_cmd = MagicMock(return_value='')

        with patch.dict(pkgng.__salt__, {'cmd.run_stdout': pkg_cmd}):
            result = pkgng.list_upgrades(refresh=False)
            self.assertDictEqual(result, {})
            pkg_cmd.assert_called_with(
                ['pkg', 'upgrade', '--dry-run', '--quiet', '--no-repo-update'],
                output_loglevel='trace', python_shell=False, ignore_retcode=True
            )



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Gareth J. Greenaway <gareth@saltstack.com>`
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import datetime
import time

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.gpg as gpg
import salt.utils.path


try:
    import gnupg  # pylint: disable=import-error,unused-import
    HAS_GPG = True
except ImportError:
    HAS_GPG = False


@skipIf(not salt.utils.path.which('gpg'), 'GPG not installed. Skipping')
@skipIf(NO_MOCK, NO_MOCK_REASON)
class GpgTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.gpg
    '''
    def setup_loader_modules(self):
        return {gpg: {'__salt__': {}}}

    @skipIf(not HAS_GPG, 'GPG Module Unavailable')
    def test_list_keys(self):
        '''
        Test gpg.list_keys
        '''

        _user_mock = {u'shell': u'/bin/bash',
                      u'workphone': u'',
                      u'uid': 0,
                      u'passwd': u'x',
                      u'roomnumber': u'',
                      u'gid': 0,
                      u'groups': [
                        u'root'
                      ],
                      u'home': u'/root',
                      u'fullname': u'root',
                      u'homephone': u'',
                      u'name': u'root'}

        _list_result = [{u'dummy': u'',
                         u'keyid': u'xxxxxxxxxxxxxxxx',
                         u'expires': u'2011188692',
                         u'sigs': [],
                         u'subkeys': [[u'xxxxxxxxxxxxxxxx', u'e', u'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx']],
                         u'length': u'4096',
                         u'ownertrust': u'-',
                         u'sig': u'',
                         u'algo': u'1',
                         u'fingerprint': u'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx',
                         u'date': u'1506612692',
                         u'trust': u'-',
                         u'type': u'pub',
                         u'uids': [u'GPG Person <person@example.com>']}]

        _expected_result = [{u'keyid': u'xxxxxxxxxxxxxxxx',
                             u'uids': [u'GPG Person <person@example.com>'],
                             u'created': '2017-09-28',
                             u'expires': '2033-09-24',
                             u'keyLength': u'4096',
                             u'ownerTrust': u'Unknown',
                             u'fingerprint': u'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx',
                             u'trust': u'Unknown'}]

        mock_opt = MagicMock(return_value='root')
        with patch.dict(gpg.__salt__, {'user.info': MagicMock(return_value=_user_mock)}):
            with patch.dict(gpg.__salt__, {'config.option': mock_opt}):
                with patch.object(gpg, '_list_keys', return_value=_list_result):
                    self.assertEqual(gpg.list_keys(), _expected_result)

    @skipIf(not HAS_GPG, 'GPG Module Unavailable')
    def test_get_key(self):
        '''
        Test gpg.get_key
        '''

        _user_mock = {u'shell': u'/bin/bash',
                      u'workphone': u'',
                      u'uid': 0,
                      u'passwd': u'x',
                      u'roomnumber': u'',
                      u'gid': 0,
                      u'groups': [
                        u'root'
                      ],
                      u'home': u'/root',
                      u'fullname': u'root',
                      u'homephone': u'',
                      u'name': u'root'}

        _list_result = [{u'dummy': u'',
                         u'keyid': u'xxxxxxxxxxxxxxxx',
                         u'expires': u'2011188692',
                         u'sigs': [],
                         u'subkeys': [[u'xxxxxxxxxxxxxxxx', u'e', u'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx']],
                         u'length': u'4096',
                         u'ownertrust': u'-',
                         u'sig': u'',
                         u'algo': u'1',
                         u'fingerprint': u'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx',
                         u'date': u'1506612692',
                         u'trust': u'-',
                         u'type': u'pub',
                         u'uids': [u'GPG Person <person@example.com>']}]

        _expected_result = {u'fingerprint': u'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx',
                            u'keyid': u'xxxxxxxxxxxxxxxx',
                            u'uids': [u'GPG Person <person@example.com>'],
                            u'created': u'2017-09-28',
                            u'trust': u'Unknown',
                            u'ownerTrust': u'Unknown',
                            u'expires': u'2033-09-24',
                            u'keyLength': u'4096'}

        mock_opt = MagicMock(return_value='root')
        with patch.dict(gpg.__salt__, {'user.info': MagicMock(return_value=_user_mock)}):
            with patch.dict(gpg.__salt__, {'config.option': mock_opt}):
                with patch.object(gpg, '_list_keys', return_value=_list_result):
                    ret = gpg.get_key('xxxxxxxxxxxxxxxx')
                    self.assertEqual(ret, _expected_result)

    @skipIf(not HAS_GPG, 'GPG Module Unavailable')
    def test_delete_key(self):
        '''
        Test gpg.delete_key
        '''

        _user_mock = {u'shell': u'/bin/bash',
                      u'workphone': u'',
                      u'uid': 0,
                      u'passwd': u'x',
                      u'roomnumber': u'',
                      u'gid': 0,
                      u'groups': [
                        u'root'
                      ],
                      u'home': u'/root',
                      u'fullname': u'root',
                      u'homephone': u'',
                      u'name': u'root'}

        _list_result = [{'dummy': u'',
                         'keyid': u'xxxxxxxxxxxxxxxx',
                         'expires': u'2011188692',
                         'sigs': [],
                         'subkeys': [[u'xxxxxxxxxxxxxxxx', u'e', u'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx']],
                         'length': u'4096',
                         'ownertrust': u'-',
                         'sig': u'',
                         'algo': u'1',
                         'fingerprint': u'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx',
                         'date': u'1506612692',
                         'trust': u'-',
                         'type': u'pub',
                         'uids': [u'GPG Person <person@example.com>']}]

        _expected_result = {u'res': True,
                            u'message': u'Secret key for xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx deleted\nPublic key for xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx deleted'}

        mock_opt = MagicMock(return_value='root')
        with patch.dict(gpg.__salt__, {'user.info': MagicMock(return_value=_user_mock)}):
            with patch.dict(gpg.__salt__, {'config.option': mock_opt}):
                with patch.object(gpg, '_list_keys', return_value=_list_result):
                    with patch('salt.modules.gpg.gnupg.GPG.delete_keys', MagicMock(return_value='ok')):
                        ret = gpg.delete_key('xxxxxxxxxxxxxxxx', delete_secret=True)
                        self.assertEqual(ret, _expected_result)

    @skipIf(not HAS_GPG, 'GPG Module Unavailable')
    def test_search_keys(self):
        '''
        Test gpg.search_keys
        '''

        _user_mock = {'shell': '/bin/bash',
                      'workphone': '',
                      'uid': 0,
                      'passwd': 'x',
                      'roomnumber': '',
                      'gid': 0,
                      'groups': [
                        'root'
                      ],
                      'home': '/root',
                      'fullname': 'root',
                      'homephone': '',
                      'name': 'root'}

        _search_result = [{u'keyid': u'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx',
                           u'uids': [u'GPG Person <person@example.com>'],
                           u'expires': u'',
                           u'sigs': [],
                           u'length': u'1024',
                           u'algo': u'17',
                           u'date': int(time.mktime(datetime.datetime(2004, 11, 13).timetuple())),
                           u'type': u'pub'}]

        _expected_result = [{u'uids': [u'GPG Person <person@example.com>'],
                             'created': '2004-11-13',
                             u'keyLength': u'1024',
                             u'keyid': u'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx'}]

        mock_opt = MagicMock(return_value='root')
        with patch.dict(gpg.__salt__, {'user.info': MagicMock(return_value=_user_mock)}):
            with patch.dict(gpg.__salt__, {'config.option': mock_opt}):
                with patch.object(gpg, '_search_keys', return_value=_search_result):
                    ret = gpg.search_keys('person@example.com')
                    self.assertEqual(ret, _expected_result)



# -*- coding: utf-8 -*-
'''
Beacon that fires events on image import/delete.

.. code-block:: yaml

    ## minimal
    # - check for new images every 1 second (salt default)
    # - does not send events at startup
    beacons:
      imgadm: []

    ## standard
    # - check for new images every 60 seconds
    # - send import events at startup for all images
    beacons:
      imgadm:
        - interval: 60
        - startup_import_event: True
'''

# Import Python libs
from __future__ import absolute_import, unicode_literals
import logging

# Import 3rd-party libs
# pylint: disable=import-error
from salt.ext.six.moves import map
# pylint: enable=import-error

__virtualname__ = 'imgadm'
IMGADM_STATE = {
  'first_run': True,
  'images': [],
}

log = logging.getLogger(__name__)


def __virtual__():
    '''
    Provides imgadm beacon on SmartOS
    '''
    if 'imgadm.list' in __salt__:
        return True
    else:
        return (
            False,
            '{0} beacon can only be loaded on SmartOS compute nodes'.format(
                __virtualname__
            )
        )


def validate(config):
    '''
    Validate the beacon configuration
    '''
    vcfg_ret = True
    vcfg_msg = 'Valid beacon configuration'

    if not isinstance(config, list):
        vcfg_ret = False
        vcfg_msg = 'Configuration for imgadm beacon must be a list!'

    return vcfg_ret, vcfg_msg


def beacon(config):
    '''
    Poll imgadm and compare available images
    '''
    ret = []

    # NOTE: lookup current images
    current_images = __salt__['imgadm.list'](verbose=True)

    # NOTE: apply configuration
    if IMGADM_STATE['first_run']:
        log.info('Applying configuration for imgadm beacon')

        _config = {}
        list(map(_config.update, config))

        if 'startup_import_event' not in _config or not _config['startup_import_event']:
            IMGADM_STATE['images'] = current_images

    # NOTE: import events
    for uuid in current_images:
        event = {}
        if uuid not in IMGADM_STATE['images']:
            event['tag'] = "imported/{}".format(uuid)
            for label in current_images[uuid]:
                event[label] = current_images[uuid][label]

        if event:
            ret.append(event)

    # NOTE: delete events
    for uuid in IMGADM_STATE['images']:
        event = {}
        if uuid not in current_images:
            event['tag'] = "deleted/{}".format(uuid)
            for label in IMGADM_STATE['images'][uuid]:
                event[label] = IMGADM_STATE['images'][uuid][label]

        if event:
            ret.append(event)

    # NOTE: update stored state
    IMGADM_STATE['images'] = current_images

    # NOTE: disable first_run
    if IMGADM_STATE['first_run']:
        IMGADM_STATE['first_run'] = False

    return ret

# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4



# -*- coding: utf-8 -*-
'''
Test AnsibleGate State Module
'''
from __future__ import absolute_import, unicode_literals, print_function

# Import python libraries
import os
import shutil
import tempfile
import yaml

# Import salt libraries
import salt.utils.files
import salt.utils.path

# Import testing libraries
from tests.support.case import ModuleCase
from tests.support.helpers import (
    destructiveTest,
    requires_sshd_server,
    requires_system_grains,
    flaky
)
from tests.support.mixins import SaltReturnAssertsMixin
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import skipIf


@destructiveTest
@requires_sshd_server
@skipIf(not salt.utils.path.which('ansible-playbook'), 'ansible-playbook is not installed')
class AnsiblePlaybooksTestCase(ModuleCase, SaltReturnAssertsMixin):
    '''
    Test ansible.playbooks states
    '''

    @requires_system_grains
    def setUp(self, grains=None):
        if grains.get('os_family') == 'RedHat' and grains.get('osmajorrelease') == 6:
            self.skipTest('This test hangs the test suite on RedHat 6. Skipping for now.')

        priv_file = os.path.join(RUNTIME_VARS.TMP_CONF_DIR, 'key_test')
        data = {
            'all': {
                'hosts': {
                    'localhost': {
                        'ansible_host': '127.0.0.1',
                        'ansible_port': 2827,
                        'ansible_user': RUNTIME_VARS.RUNNING_TESTS_USER,
                        'ansible_ssh_private_key_file': priv_file,
                        'ansible_ssh_extra_args': (
                            '-o StrictHostKeyChecking=false '
                            '-o UserKnownHostsFile=/dev/null '
                        )
                    },
                },
            },
        }
        self.tempdir = tempfile.mkdtemp()
        self.inventory = self.tempdir + 'inventory'
        with salt.utils.files.fopen(self.inventory, 'w') as yaml_file:
            yaml.dump(data, yaml_file, default_flow_style=False)

    def tearDown(self):
        shutil.rmtree(self.tempdir)
        delattr(self, 'tempdir')
        delattr(self, 'inventory')

    @flaky
    def test_ansible_playbook(self):
        ret = self.run_state(
            'ansible.playbooks',
            name='remove.yml',
            git_repo='git://github.com/gtmanfred/playbooks.git',
            ansible_kwargs={'inventory': self.inventory}
        )
        self.assertSaltTrueReturn(ret)
        ret = self.run_state(
            'ansible.playbooks',
            name='install.yml',
            git_repo='git://github.com/gtmanfred/playbooks.git',
            ansible_kwargs={'inventory': self.inventory}
        )
        self.assertSaltTrueReturn(ret)



# -*- coding: utf-8 -*-
'''
Control the OpenBSD packet filter (PF).

:codeauthor: Jasper Lievisse Adriaanse <j@jasper.la>

.. versionadded:: 2019.2.0
'''

from __future__ import absolute_import, print_function, unicode_literals

# Import python libs
import logging
import re

# Import salt libs
import salt.utils.path
from salt.exceptions import (CommandExecutionError, SaltInvocationError)

log = logging.getLogger(__name__)


def __virtual__():
    '''
    Only works on OpenBSD for now; other systems with pf (macOS, FreeBSD, etc)
    need to be tested before enabling them.
    '''
    if __grains__['os'] == 'OpenBSD' and salt.utils.path.which('pfctl'):
        return True

    return (False, 'The pf execution module cannot be loaded: either the system is not OpenBSD or the pfctl binary was not found')


def enable():
    '''
    Enable the Packet Filter.

    CLI example:

    .. code-block:: bash

        salt '*' pf.enable
    '''
    ret = {}
    result = __salt__['cmd.run_all']('pfctl -e',
                                     output_loglevel='trace',
                                     python_shell=False)

    if result['retcode'] == 0:
        ret = {'comment': 'pf enabled', 'changes': True}
    else:
        # If pf was already enabled the return code is also non-zero.
        # Don't raise an exception in that case.
        if result['stderr'] == 'pfctl: pf already enabled':
            ret = {'comment': 'pf already enabled', 'changes': False}
        else:
            raise CommandExecutionError(
                'Could not enable pf',
                info={'errors': [result['stderr']], 'changes': False}
            )

    return ret


def disable():
    '''
    Disable the Packet Filter.

    CLI example:

    .. code-block:: bash

        salt '*' pf.disable
    '''
    ret = {}
    result = __salt__['cmd.run_all']('pfctl -d',
                                     output_loglevel='trace',
                                     python_shell=False)

    if result['retcode'] == 0:
        ret = {'comment': 'pf disabled', 'changes': True}
    else:
        # If pf was already disabled the return code is also non-zero.
        # Don't raise an exception in that case.
        if result['stderr'] == 'pfctl: pf not enabled':
            ret = {'comment': 'pf already disabled', 'changes': False}
        else:
            raise CommandExecutionError(
                'Could not disable pf',
                info={'errors': [result['stderr']], 'changes': False}
            )

    return ret


def loglevel(level):
    '''
    Set the debug level which limits the severity of log messages printed by ``pf(4)``.

    level:
        Log level. Should be one of the following: emerg, alert, crit, err, warning, notice,
        info or debug.

    CLI example:

    .. code-block:: bash

        salt '*' pf.loglevel emerg
    '''
    # There's no way to getting the previous loglevel so imply we've
    # always made a change.
    ret = {'changes': True}

    all_levels = ['emerg', 'alert', 'crit', 'err', 'warning', 'notice', 'info', 'debug']
    if level not in all_levels:
        raise SaltInvocationError('Unknown loglevel: {0}'.format(level))

    result = __salt__['cmd.run_all']('pfctl -x {0}'.format(level),
                                     output_loglevel='trace',
                                     python_shell=False)

    if result['retcode'] != 0:
        raise CommandExecutionError(
            'Problem encountered setting loglevel',
            info={'errors': [result['stderr']], 'changes': False}
        )

    return ret


def load(file='/etc/pf.conf', noop=False):
    '''
    Load a ruleset from the specific file, overwriting the currently loaded ruleset.

    file:
        Full path to the file containing the ruleset.

    noop:
        Don't actually load the rules, just parse them.

    CLI example:

    .. code-block:: bash

        salt '*' pf.load /etc/pf.conf.d/lockdown.conf
    '''
    # We cannot precisely determine if loading the ruleset implied
    # any changes so assume it always does.
    ret = {'changes': True}
    cmd = ['pfctl', '-f', file]

    if noop:
        ret['changes'] = False
        cmd.append('-n')

    result = __salt__['cmd.run_all'](cmd,
                                     output_loglevel='trace',
                                     python_shell=False)

    if result['retcode'] != 0:
        raise CommandExecutionError(
            'Problem loading the ruleset from {0}'.format(file),
            info={'errors': [result['stderr']], 'changes': False}
        )

    return ret


def flush(modifier):
    '''
    Flush the specified packet filter parameters.

    modifier:
        Should be one of the following:

        - all
        - info
        - osfp
        - rules
        - sources
        - states
        - tables

        Please refer to the OpenBSD `pfctl(8) <https://man.openbsd.org/pfctl#T>`_
        documentation for a detailed explanation of each command.

    CLI example:

    .. code-block:: bash

        salt '*' pf.flush states
    '''
    ret = {}

    all_modifiers = ['rules', 'states', 'info', 'osfp', 'all', 'sources', 'tables']

    # Accept the following two modifiers to allow for a consistent interface between
    # pfctl(8) and Salt.
    capital_modifiers = ['Sources', 'Tables']
    all_modifiers += capital_modifiers
    if modifier.title() in capital_modifiers:
        modifier = modifier.title()

    if modifier not in all_modifiers:
        raise SaltInvocationError('Unknown modifier: {0}'.format(modifier))

    cmd = 'pfctl -v -F {0}'.format(modifier)
    result = __salt__['cmd.run_all'](cmd,
                                     output_loglevel='trace',
                                     python_shell=False)

    if result['retcode'] == 0:
        if re.match(r'^0.*', result['stderr']):
            ret['changes'] = False
        else:
            ret['changes'] = True

        ret['comment'] = result['stderr']
    else:
        raise CommandExecutionError(
            'Could not flush {0}'.format(modifier),
            info={'errors': [result['stderr']], 'changes': False}
        )

    return ret


def table(command, table, **kwargs):
    '''
    Apply a command on the specified table.

    table:
        Name of the table.

    command:
        Command to apply to the table. Supported commands are:

        - add
        - delete
        - expire
        - flush
        - kill
        - replace
        - show
        - test
        - zero

        Please refer to the OpenBSD `pfctl(8) <https://man.openbsd.org/pfctl#T>`_
        documentation for a detailed explanation of each command.

    CLI example:

    .. code-block:: bash

        salt '*' pf.table expire table=spam_hosts number=300
        salt '*' pf.table add table=local_hosts addresses='["127.0.0.1", "::1"]'
    '''
    ret = {}

    all_commands = ['kill', 'flush', 'add', 'delete', 'expire', 'replace', 'show', 'test', 'zero']
    if command not in all_commands:
        raise SaltInvocationError('Unknown table command: {0}'.format(command))

    cmd = ['pfctl', '-t', table, '-T', command]

    if command in ['add', 'delete', 'replace', 'test']:
        cmd += kwargs.get('addresses', [])
    elif command == 'expire':
        number = kwargs.get('number', None)
        if not number:
            raise SaltInvocationError('need expire_number argument for expire command')
        else:
            cmd.append(number)

    result = __salt__['cmd.run_all'](cmd,
                                     output_level='trace',
                                     python_shell=False)

    if result['retcode'] == 0:
        if command == 'show':
            ret = {'comment': result['stdout'].split()}
        elif command == 'test':
            ret = {'comment': result['stderr'], 'matches': True}
        else:
            if re.match(r'^(0.*|no changes)', result['stderr']):
                ret['changes'] = False
            else:
                ret['changes'] = True

            ret['comment'] = result['stderr']
    else:
        # 'test' returns a non-zero code if the address didn't match, even if
        # the command itself ran fine; also set 'matches' to False since not
        # everything matched.
        if command == 'test' and re.match(r'^\d+/\d+ addresses match.$', result['stderr']):
            ret = {'comment': result['stderr'], 'matches': False}
        else:
            raise CommandExecutionError(
                'Could not apply {0} on table {1}'.format(command, table),
                info={'errors': [result['stderr']], 'changes': False}
            )

    return ret


def show(modifier):
    '''
    Show filter parameters.

    modifier:
        Modifier to apply for filtering. Only a useful subset of what pfctl supports
        can be used with Salt.

        - rules
        - states
        - tables

    CLI example:

    .. code-block:: bash

        salt '*' pf.show rules
    '''
    # By definition showing the parameters makes no changes.
    ret = {'changes': False}

    capital_modifiers = ['Tables']
    all_modifiers = ['rules', 'states', 'tables']
    all_modifiers += capital_modifiers
    if modifier.title() in capital_modifiers:
        modifier = modifier.title()

    if modifier not in all_modifiers:
        raise SaltInvocationError('Unknown modifier: {0}'.format(modifier))

    cmd = 'pfctl -s {0}'.format(modifier)
    result = __salt__['cmd.run_all'](cmd,
                                     output_loglevel='trace',
                                     python_shell=False)

    if result['retcode'] == 0:
        ret['comment'] = result['stdout'].split('\n')
    else:
        raise CommandExecutionError(
            'Could not show {0}'.format(modifier),
            info={'errors': [result['stderr']], 'changes': False}
        )

    return ret



# coding: utf-8

# Python libs
from __future__ import absolute_import
import datetime
import logging

# Salt testing libs
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, patch, MagicMock, mock_open
from tests.support.mixins import LoaderModuleMockMixin

# Salt libs
import salt.beacons.wtmp as wtmp
from salt.ext import six

# pylint: disable=import-error
try:
    import dateutil.parser as dateutil_parser  # pylint: disable=unused-import
    _TIME_SUPPORTED = True
except ImportError:
    _TIME_SUPPORTED = False

raw = b'\x07\x00\x00\x00H\x18\x00\x00pts/14\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00s/14gareth\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00::1\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x13I\xc5YZf\x05\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
pack = (7, 6216, b'pts/14\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00', b's/14', b'gareth\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00', b'::1\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00', 0, 0, 0, 1506101523, 353882, 0, 0, 0, 16777216)

log = logging.getLogger(__name__)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class WTMPBeaconTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test case for salt.beacons.[s]
    '''

    def setup_loader_modules(self):
        return {
            wtmp: {
                '__context__': {'wtmp.loc': 2},
                '__salt__': {},
            }
        }

    def test_non_list_config(self):
        config = {}
        ret = wtmp.validate(config)

        self.assertEqual(ret, (False, 'Configuration for wtmp beacon must'
                                      ' be a list.'))

    def test_empty_config(self):
        config = [{}]

        ret = wtmp.validate(config)

        self.assertEqual(ret, (True, 'Valid beacon configuration'))

    def test_no_match(self):
        config = [{'users': {'gareth': {'time_range': {'end': '09-22-2017 5pm',
                                                       'start': '09-22-2017 3pm'}}}}
                  ]

        ret = wtmp.validate(config)

        self.assertEqual(ret, (True, 'Valid beacon configuration'))

        with patch('salt.utils.files.fopen', mock_open(b'')) as m_open:
            ret = wtmp.beacon(config)
            call_args = next(six.itervalues(m_open.filehandles))[0].call.args
            assert call_args == (wtmp.WTMP, 'rb'), call_args
            assert ret == [], ret

    def test_invalid_users(self):
        config = [{'users': ['gareth']}]

        ret = wtmp.validate(config)

        self.assertEqual(ret, (False, 'User configuration for wtmp beacon must be a dictionary.'))

    def test_invalid_groups(self):
        config = [{'groups': ['docker']}]

        ret = wtmp.validate(config)

        self.assertEqual(ret, (False, 'Group configuration for wtmp beacon must be a dictionary.'))

    def test_default_invalid_time_range(self):
        config = [{'defaults': {'time_range': {'start': '3pm'}}}]

        ret = wtmp.validate(config)

        self.assertEqual(ret, (False, 'The time_range parameter for wtmp beacon must contain start & end options.'))

    def test_users_invalid_time_range(self):
        config = [{'users': {'gareth': {'time_range': {'start': '3pm'}}}}]

        ret = wtmp.validate(config)

        self.assertEqual(ret, (False, 'The time_range parameter for wtmp beacon must contain start & end options.'))

    def test_groups_invalid_time_range(self):
        config = [{'groups': {'docker': {'time_range': {'start': '3pm'}}}}]

        ret = wtmp.validate(config)

        self.assertEqual(ret, (False, 'The time_range parameter for wtmp beacon must contain start & end options.'))

    def test_match(self):
        with patch('salt.utils.files.fopen',
                   mock_open(read_data=raw)):
            with patch('struct.unpack',
                       MagicMock(return_value=pack)):
                config = [{'users': {'gareth': {}}}]

                ret = wtmp.validate(config)

                self.assertEqual(ret, (True, 'Valid beacon configuration'))

                _expected = [{'PID': 6216,
                              'action': 'login',
                              'line': 'pts/14',
                              'session': 0,
                              'time': 0,
                              'exit_status': 0,
                              'inittab': 's/14',
                              'type': 7,
                              'addr': 1506101523,
                              'hostname': '::1',
                              'user': 'gareth'}]

                ret = wtmp.beacon(config)
                log.debug('{}'.format(ret))
                self.assertEqual(ret, _expected)

    @skipIf(not _TIME_SUPPORTED, 'dateutil.parser is missing.')
    def test_match_time(self):
        with patch('salt.utils.files.fopen',
                   mock_open(read_data=raw)):
            mock_now = datetime.datetime(2017, 9, 22, 16, 0, 0, 0)
            with patch('datetime.datetime', MagicMock()), \
                    patch('datetime.datetime.now',
                          MagicMock(return_value=mock_now)):
                with patch('struct.unpack',
                           MagicMock(return_value=pack)):
                    config = [{'users': {'gareth': {'time': {'end': '09-22-2017 5pm',
                                                             'start': '09-22-2017 3pm'}}}}
                              ]

                    ret = wtmp.validate(config)

                    self.assertEqual(ret, (True, 'Valid beacon configuration'))

                    _expected = [{'PID': 6216,
                                  'action': 'login',
                                  'line': 'pts/14',
                                  'session': 0,
                                  'time': 0,
                                  'exit_status': 0,
                                  'inittab': 's/14',
                                  'type': 7,
                                  'addr': 1506101523,
                                  'hostname': '::1',
                                  'user': 'gareth'}]

                    ret = wtmp.beacon(config)
                    self.assertEqual(ret, _expected)

    def test_match_group(self):

        for groupadd in ('salt.modules.aix_group',
                         'salt.modules.mac_group',
                         'salt.modules.pw_group',
                         'salt.modules.solaris_group',
                         'salt.modules.win_groupadd'):
            mock_group_info = {'passwd': 'x',
                               'gid': 100,
                               'name': 'users',
                               'members': ['gareth']}

            with patch('salt.utils.files.fopen',
                       mock_open(read_data=raw)):
                with patch('time.time',
                           MagicMock(return_value=1506121200)):
                    with patch('struct.unpack',
                               MagicMock(return_value=pack)):
                        with patch('{0}.info'.format(groupadd),
                                   new=MagicMock(return_value=mock_group_info)):
                            config = [{'group': {'users': {'time': {'end': '09-22-2017 5pm',
                                                                    'start': '09-22-2017 3pm'}}}}
                                      ]

                            ret = wtmp.validate(config)

                            self.assertEqual(ret,
                                             (True, 'Valid beacon configuration'))

                            _expected = [{'PID': 6216,
                                          'action': 'login',
                                          'line': 'pts/14',
                                          'session': 0,
                                          'time': 0,
                                          'exit_status': 0,
                                          'inittab': 's/14',
                                          'type': 7,
                                          'addr': 1506101523,
                                          'hostname': '::1',
                                          'user': 'gareth'}]

                            ret = wtmp.beacon(config)
                            self.assertEqual(ret, _expected)



# -*- coding: utf-8 -*-
#
# Author: Bo Maryniuk <bo@suse.de>
#
# Copyright 2017 SUSE LLC
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

r'''
Execution of Ansible modules from within states
===============================================

With `ansible.call` these states allow individual Ansible module calls to be
made via states. To call an Ansible module function use a :mod:`module.run <salt.states.ansible.call>`
state:

.. code-block:: yaml

    some_set_of_tasks:
      ansible:
        - system.ping
        - packaging.os.zypper
          - name: emacs
          - state: installed

'''
from __future__ import absolute_import, print_function, unicode_literals
import logging
import os
import sys

# Import salt modules
import salt.fileclient
import salt.ext.six as six
from salt.utils.decorators import depends
import salt.utils.decorators.path

log = logging.getLogger(__name__)
__virtualname__ = 'ansible'


@depends('ansible')
class AnsibleState(object):
    '''
    Ansible state caller.
    '''
    def get_args(self, argset):
        '''
        Get args and kwargs from the argset.

        :param argset:
        :return:
        '''
        args = []
        kwargs = {}
        for element in argset or []:
            if isinstance(element, dict):
                kwargs.update(element)
            else:
                args.append(element)
        return args, kwargs

    def __call__(self, **kwargs):
        '''
        Call Ansible module.

        :return:
        '''

        ret = {
            'name': kwargs.pop('name'),
            'changes': {},
            'comment': '',
            'result': True,
        }

        for mod_name, mod_params in kwargs.items():
            args, kwargs = self.get_args(mod_params)
            try:
                ans_mod_out = __salt__['ansible.{0}'.format(mod_name)](**{'__pub_arg': [args, kwargs]})
            except Exception as err:
                ans_mod_out = 'Module "{0}" failed. Error message: ({1}) {2}'.format(
                    mod_name, err.__class__.__name__, err)
                ret['result'] = False
            ret['changes'][mod_name] = ans_mod_out

        return ret


def __virtual__():
    '''
    Disable, if Ansible is not available around on the Minion.
    '''
    setattr(sys.modules[__name__], 'call', lambda **kwargs: AnsibleState()(**kwargs))   # pylint: disable=W0108
    return __virtualname__


def _client():
    '''
    Get a fileclient
    '''
    return salt.fileclient.get_file_client(__opts__)


def _changes(plays):
    '''
    Find changes in ansible return data
    '''
    changes = {}
    for play in plays['plays']:
        task_changes = {}
        for task in play['tasks']:
            host_changes = {}
            for host, data in six.iteritems(task['hosts']):
                if data['changed'] is True:
                    host_changes[host] = data.get('diff', data.get('changes', {}))
            if host_changes:
                task_changes[task['task']['name']] = host_changes
        if task_changes:
            changes[play['play']['name']] = task_changes
    return changes


@salt.utils.decorators.path.which('ansible-playbook')
def playbooks(name, rundir=None, git_repo=None, git_kwargs=None, ansible_kwargs=None):
    '''
    Run Ansible Playbooks

    :param name: path to playbook. This can be relative to rundir or the git repo
    :param rundir: location to run ansible-playbook from.
    :param git_repo: git repository to clone for ansible playbooks.  This is cloned
                     using the `git.latest` state, and is cloned to the `rundir`
                     if specified, otherwise it is clone to the `cache_dir`
    :param git_kwargs: extra kwargs to pass to `git.latest` state module besides
                       the `name` and `target`
    :param ansible_kwargs: extra kwargs to pass to `ansible.playbooks` execution
                           module besides the `name` and `target`

    :return: Ansible playbook output.

    .. code-block:: yaml

        run nginx install:
          ansible.playbooks:
            - name: install.yml
            - git_repo: git://github.com/gituser/playbook.git
            - git_kwargs:
                rev: master
    '''
    ret = {
        'result': False,
        'changes': {},
        'comment': 'Running playbook {0}'.format(name),
        'name': name,
    }
    if git_repo:
        if not isinstance(rundir, six.text_type) or not os.path.isdir(rundir):
            rundir = _client()._extrn_path(git_repo, 'base')
            log.trace('rundir set to %s', rundir)
        if not isinstance(git_kwargs, dict):
            log.debug('Setting git_kwargs to empty dict: %s', git_kwargs)
            git_kwargs = {}
        __states__['git.latest'](
            name=git_repo,
            target=rundir,
            **git_kwargs
        )
    if not isinstance(ansible_kwargs, dict):
        log.debug('Setting ansible_kwargs to empty dict: %s', ansible_kwargs)
        ansible_kwargs = {}
    checks = __salt__['ansible.playbooks'](name, rundir=rundir, check=True, diff=True, **ansible_kwargs)
    if all(not check['changed'] for check in six.itervalues(checks['stats'])):
        ret['comment'] = 'No changes to be made from playbook {0}'.format(name)
        ret['result'] = True
    elif __opts__['test']:
        ret['comment'] = 'Changes will be made from playbook {0}'.format(name)
        ret['result'] = None
        ret['changes'] = _changes(checks)
    else:
        results = __salt__['ansible.playbooks'](name, rundir=rundir, diff=True, **ansible_kwargs)
        ret['comment'] = 'Changes were made by playbook {0}'.format(name)
        ret['changes'] = _changes(results)
        ret['result'] = all(
            not check['failures'] and not check['unreachable']
            for check in six.itervalues(checks['stats'])
        )
    return ret



# -*- coding: utf-8 -*-
'''
DRBD administration module
'''
from __future__ import absolute_import, print_function, unicode_literals

import logging

log = logging.getLogger(__name__)


def _analyse_overview_field(content):
    if "(" in content:
        # Output like "Connected(2*)" or "UpToDate(2*)"
        return content.split("(")[0], content.split("(")[0]
    elif "/" in content:
        # Output like "Primar/Second" or "UpToDa/UpToDa"
        return content.split("/")[0], content.split("/")[1]

    return content, ""


def overview():
    '''
    Show status of the DRBD devices, support two nodes only.

    CLI Example:

    .. code-block:: bash

        salt '*' drbd.overview
    '''
    cmd = 'drbd-overview'
    for line in __salt__['cmd.run'](cmd).splitlines():
        ret = {}
        fields = line.strip().split()
        minnum = fields[0].split(':')[0]
        device = fields[0].split(':')[1]
        connstate, _ = _analyse_overview_field(fields[1])
        localrole, partnerrole = _analyse_overview_field(fields[2])
        localdiskstate, partnerdiskstate = _analyse_overview_field(fields[3])
        if localdiskstate.startswith("UpTo"):
            if partnerdiskstate.startswith("UpTo"):
                if len(fields) >= 5:
                    mountpoint = fields[4]
                    fs_mounted = fields[5]
                    totalsize = fields[6]
                    usedsize = fields[7]
                    remainsize = fields[8]
                    perc = fields[9]
                    ret = {
                        'minor number': minnum,
                        'device': device,
                        'connection state': connstate,
                        'local role': localrole,
                        'partner role': partnerrole,
                        'local disk state': localdiskstate,
                        'partner disk state': partnerdiskstate,
                        'mountpoint': mountpoint,
                        'fs': fs_mounted,
                        'total size': totalsize,
                        'used': usedsize,
                        'remains': remainsize,
                        'percent': perc,
                    }
                else:
                    ret = {
                        'minor number': minnum,
                        'device': device,
                        'connection state': connstate,
                        'local role': localrole,
                        'partner role': partnerrole,
                        'local disk state': localdiskstate,
                        'partner disk state': partnerdiskstate,
                    }
            else:
                syncbar = fields[4]
                synced = fields[6]
                syncedbytes = fields[7]
                sync = synced+syncedbytes
                ret = {
                    'minor number': minnum,
                    'device': device,
                    'connection state': connstate,
                    'local role': localrole,
                    'partner role': partnerrole,
                    'local disk state': localdiskstate,
                    'partner disk state': partnerdiskstate,
                    'synchronisation: ': syncbar,
                    'synched': sync,
                }
    return ret



# -*- coding: utf-8 -*-
'''
This is the default data matcher.
'''
from __future__ import absolute_import, print_function, unicode_literals

import fnmatch
import logging
from salt.ext import six  # pylint: disable=3rd-party-module-not-gated

import salt.utils.data  # pylint: disable=3rd-party-module-not-gated
import salt.utils.minions  # pylint: disable=3rd-party-module-not-gated
import salt.utils.network  # pylint: disable=3rd-party-module-not-gated
import salt.loader  # pylint: disable=3rd-party-module-not-gated

log = logging.getLogger(__name__)


def match(tgt, functions=None):
    '''
    Match based on the local data store on the minion
    '''
    if functions is None:
        utils = salt.loader.utils(__opts__)
        functions = salt.loader.minion_mods(__opts__, utils=utils)
    comps = tgt.split(':')
    if len(comps) < 2:
        return False
    val = functions['data.getval'](comps[0])
    if val is None:
        # The value is not defined
        return False
    if isinstance(val, list):
        # We are matching a single component to a single list member
        for member in val:
            if fnmatch.fnmatch(six.text_type(member).lower(), comps[1].lower()):
                return True
        return False
    if isinstance(val, dict):
        if comps[1] in val:
            return True
        return False
    return bool(fnmatch.fnmatch(
        val,
        comps[1],
    ))
